@@ -484,6 +484,7 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 	gint           duration;
 	gint           work;
 	gint           complete;
+	gint           priority;
 	MrpTaskType    type;
 	MrpTaskSched   sched;
 	GList         *predecessors, *l;
@@ -512,6 +513,7 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 		      "work", &work,
 		      "constraint", &constraint,
 		      "percent-complete", &complete,
+		      "priority", &priority,
 		      "type", &type,
 		      "sched", &sched,
 		      NULL);
@@ -536,6 +538,8 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 
 	mpp_xml_set_int (node, "percent-complete", complete);
 
+	mpp_xml_set_int (node, "priority", priority);
+
 	mpp_xml_set_task_type (node, "type", type);
 
 	mpp_xml_set_task_sched (node, "scheduling", sched);
@@ -551,6 +555,9 @@ mpp_write_task_cb (MrpTask *task, MrpParser *parser)
 			mpp_write_predecessor (parser, node, l->data);
 		}
 	}
+
+	g_free (name);
+	g_free (note);
 	
 	return FALSE;
 }
@@ -586,7 +593,7 @@ mpp_write_group (MrpParser *parser, xmlNodePtr parent, MrpGroup *group)
 	
 	mpp_xml_set_int (node, "id", entry->id);
 	
-	g_object_get (G_OBJECT (group),
+	g_object_get (group,
 		      "name", &name,
 		      "manager-name", &admin_name,
 		      "manager-phone", &admin_phone,
@@ -596,7 +603,12 @@ mpp_write_group (MrpParser *parser, xmlNodePtr parent, MrpGroup *group)
 	xmlSetProp (node, "name", name);	
 	xmlSetProp (node, "admin-name", admin_name);	
 	xmlSetProp (node, "admin-phone", admin_phone);	
-	xmlSetProp (node, "admin-email", admin_email);	
+	xmlSetProp (node, "admin-email", admin_email);
+
+	g_free (name);
+	g_free (admin_name);
+	g_free (admin_phone);
+	g_free (admin_email);
 }
 
 static void
@@ -616,7 +628,7 @@ mpp_write_resource (MrpParser   *parser,
 		    MrpResource *resource)
 {
 	xmlNodePtr   node;
-	gchar       *name, *email;
+	gchar       *name, *short_name, *email;
 	gchar       *note;
 	gint         type, units;
 	gfloat       std_rate; /*, ovt_rate;*/
@@ -635,6 +647,7 @@ mpp_write_resource (MrpParser   *parser,
 
 	mrp_object_get (MRP_OBJECT (resource),
 			"name", &name,
+			"short_name", &short_name,
 			"email", &email,
 			"type", &type,
 			"units", &units,
@@ -656,6 +669,7 @@ mpp_write_resource (MrpParser   *parser,
 	mpp_xml_set_int (node, "id", resource_entry->id);
 	
 	xmlSetProp (node, "name", name);
+	xmlSetProp (node, "short-name", short_name);
 	
 	mpp_xml_set_int (node, "type", type);
 	
@@ -678,6 +692,11 @@ mpp_write_resource (MrpParser   *parser,
 	}
 
 	mpp_write_custom_properties (parser, node, MRP_OBJECT (resource));
+
+	g_free (name);
+	g_free (short_name);
+	g_free (email);
+	g_free (note);
 }
 
 static void
@@ -699,7 +718,7 @@ mpp_write_assignment (MrpParser     *parser,
 			    "allocation",
 			    NULL);
 
-	g_object_get (G_OBJECT (assignment),
+	g_object_get (assignment,
 		      "task", &task,
 		      "resource", &resource,
 		      "units", &assigned_units,
@@ -1068,8 +1087,8 @@ mrp_parser_save (MrpStorageMrproject  *module,
 	g_return_val_if_fail (MRP_IS_STORAGE_MRPROJECT (module), FALSE);
 	g_return_val_if_fail (filename != NULL && filename[0] != 0, FALSE);
 
-	if (!strstr (filename, ".mrproject")) {
-		real_filename = g_strconcat (filename, ".mrproject", NULL);
+	if (!strstr (filename, ".mrproject") && !strstr (filename, ".planner")) {
+		real_filename = g_strconcat (filename, ".planner", NULL);
 	} else {
 		real_filename = g_strdup (filename);
 	}
