@@ -43,7 +43,8 @@ enum {
 	PROP_SCHED,
 	PROP_CONSTRAINT,
 	PROP_NOTE,
-	PROP_PERCENT_COMPLETE
+	PROP_PERCENT_COMPLETE,
+	PROP_PRIORITY
 };
 
 /* Signals */
@@ -79,6 +80,9 @@ struct _MrpTaskPriv {
 	/* Percent complete, 0-100. */
 	gshort            percent_complete;
 	
+	/* Arbitary range of 0,1..9999. A hint for any (3rd party) resource leveller */
+	gint              priority;
+	
 	gchar            *name;
 	gchar            *note;
 
@@ -183,6 +187,7 @@ task_init (MrpTask *task)
 	priv->sorted_node = g_node_new (task);
 	priv->assignments = NULL;
 	priv->constraint.type = MRP_CONSTRAINT_ASAP;
+	priv->note = g_strdup ("");
 }
 
 static void
@@ -378,6 +383,15 @@ task_class_init (MrpTaskClass *klass)
 				  "Percent completed of task",
 				  0, 100, 0,
 				  G_PARAM_READWRITE));
+				  
+	g_object_class_install_property (
+		object_class,
+		PROP_PRIORITY,
+		g_param_spec_int ("priority",
+				  "Priority",
+				  "Priority of the task",
+				  0, 9999, 0,
+				  G_PARAM_READWRITE));
 }
 
 static void
@@ -541,6 +555,16 @@ task_set_property (GObject      *object,
 		
 		break;
 
+	case PROP_PRIORITY:
+		i_val = g_value_get_int (value);
+		
+		if (priv->priority != i_val) {
+			priv->priority = i_val;
+			changed = TRUE;
+		}
+		
+		break;
+		
 	default:
 		break;
 	}
@@ -602,6 +626,9 @@ task_get_property (GObject    *object,
 	case PROP_PERCENT_COMPLETE:
 		g_value_set_int (value, priv->percent_complete);
 		break;
+	case PROP_PRIORITY:
+		g_value_set_int (value, priv->priority);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -699,7 +726,7 @@ task_remove_assignments (MrpTask *task)
 	for (l = copy; l; l = l->next) {
 		assignment = l->data;
 		
-		g_signal_handlers_disconnect_by_func (assignment,
+		g_signal_handlers_disconnect_by_func (MRP_ASSIGNMENT (l->data),
 						      task_assignment_removed_cb,
 						      task);
 		g_object_unref (assignment);
@@ -727,6 +754,8 @@ task_remove_subtree_cb (GNode *node, gpointer data)
 	g_node_unlink (priv->node);
 	g_node_unlink (priv->sorted_node);
 
+	mrp_object_removed (MRP_OBJECT (task));
+
 	g_object_unref (task);
 	
 	return FALSE;
@@ -1017,8 +1046,8 @@ mrp_task_add_predecessor (MrpTask          *task,
 		
 		return NULL;
 	}
-
-	g_object_get (task, "project", &project, NULL);
+	
+	project = mrp_object_get_project (MRP_OBJECT (task));
 	manager = imrp_project_get_task_manager (project);
 	if (!mrp_task_manager_check_predecessor (manager, task, predecessor, error)) {
 		return NULL;
@@ -1569,7 +1598,7 @@ mrp_task_get_cost (MrpTask *task)
 		resource = mrp_assignment_get_resource (l->data);
 
 		mrp_object_get (resource, "cost", &cost, NULL);
-		total += mrp_assignment_get_units (l->data) * priv->work * cost / (3600.0 * 100);
+		total += mrp_assignment_get_units (l->data) * priv->duration * cost / (3600.0 * 100);
 	}
 
 	return total;
@@ -1733,7 +1762,7 @@ imrp_task_peek_successors (MrpTask *task)
 }
 
 MrpTaskType
-imrp_task_get_type (MrpTask *task)
+mrp_task_get_task_type (MrpTask *task)
 {
 	g_return_val_if_fail (MRP_IS_TASK (task), MRP_TASK_TYPE_NORMAL);
 
@@ -1741,9 +1770,25 @@ imrp_task_get_type (MrpTask *task)
 }
 
 MrpTaskSched
-imrp_task_get_sched (MrpTask *task)
+mrp_task_get_sched (MrpTask *task)
 {
 	g_return_val_if_fail (MRP_IS_TASK (task), MRP_TASK_SCHED_FIXED_WORK);
 
 	return task->priv->sched;
 }
+
+gshort
+mrp_task_get_percent_complete (MrpTask *task)
+{
+	g_return_val_if_fail (MRP_IS_TASK (task), 0);
+
+	return task->priv->percent_complete;
+}
+
+gboolean
+mrp_task_get_critical (MrpTask *task)
+{
+	g_return_val_if_fail (MRP_IS_TASK (task), FALSE);
+
+	return task->priv->critical;
+}
