@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2004 Imendio HB
  * Copyright (C) 2003 CodeFactory AB
  * Copyright (C) 2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
@@ -34,28 +35,25 @@
 #include "planner-gantt-print.h"
 #include "planner-scale-utils.h"
 
-#define d(x) 
+#define d(x)
 #define GET_PAGE(d,r,c) (&d->pages[r*d->cols_of_pages+c])
 #define INDENT_FACTOR 4
 
 typedef struct {
-	GList  *first_task;
-	GList  *last_task;
-
 	mrptime  first_time;
 	mrptime  last_time;
 
 	gdouble  gantt_x0;
 	gdouble  gantt_y0;
 
-	GList  *elements;
-	GList  *background_elements;
+	GList   *elements;
+	GList   *background_elements;
 } Page;
 
-struct _MgGanttPrintData {
+struct _PlannerGanttPrintData {
 	MrpProject        *project;
-	MgView            *view;
-	MgPrintJob        *job;
+	PlannerView       *view;
+	PlannerPrintJob   *job;
 
 	/* Used to get the visible (expanded) tasks. */
 	GtkTreeView       *tree_view;
@@ -65,11 +63,11 @@ struct _MgGanttPrintData {
 
 	gint               level;
 	
-	MgScaleUnit        major_unit;
-	MgScaleFormat      major_format;
+	PlannerScaleUnit   major_unit;
+	PlannerScaleFormat major_format;
 	
-	MgScaleUnit        minor_unit;
-	MgScaleFormat      minor_format;
+	PlannerScaleUnit   minor_unit;
+	PlannerScaleFormat minor_format;
 	
 	gdouble            header_height;
 	
@@ -97,7 +95,7 @@ struct _MgGanttPrintData {
 	GnomeFont         *font;
 	GnomeFont         *font_bold;
 
-	GList            *tasks;
+	GList             *tasks;
 
 	gdouble            f;
 	
@@ -128,7 +126,6 @@ typedef struct {
 	gdouble x, y;
 } TaskCoord;
 
-
 typedef struct {
 	MrpTask *task;
 	gint     depth;
@@ -163,13 +160,15 @@ typedef struct {
 	gdouble      x1, y1;
 	gdouble      x2, y2;
 
+	gdouble      x_complete;
+
 	gboolean     is_critical;
 	gchar       *resources;
 } Element;
 
 
 static void
-print_table_header (MgGanttPrintData *data)
+print_table_header (PlannerGanttPrintData *data)
 {
 	gdouble x, y;
 
@@ -186,32 +185,32 @@ print_table_header (MgGanttPrintData *data)
 	y = data->row_height;
 
 	planner_print_job_show_clipped (data->job,
-				   x, y,
-				   _("Name"),
-				   data->name_x1, 0, 
-				   data->name_x2 - data->job->x_pad / 2, y + data->row_height);
+					x, y,
+					_("Name"),
+					data->name_x1, 0, 
+					data->name_x2 - data->job->x_pad / 2, y + data->row_height);
 
 	x = data->work_x1 + data->job->x_pad;
 
 	planner_print_job_show_clipped (data->job,
-				   x, y,
-				   _("Work"),
-				   data->work_x1, 0, 
-				   data->work_x2 - data->job->x_pad / 2, y + data->row_height);
+					x, y,
+					_("Work"),
+					data->work_x1, 0, 
+					data->work_x2 - data->job->x_pad / 2, y + data->row_height);
 
 	planner_print_job_set_font_regular (data->job);
 }
 
 static void
-print_table_tasks (MgGanttPrintData *data,
-		   gboolean          header,
-		   GList           *tasks,
-		   gint              first)
+print_table_tasks (PlannerGanttPrintData *data,
+		   gboolean               header,
+		   GList                 *tasks,
+		   gint                   first)
 {
 	gchar     *str;
 	gint       work;
 	gdouble    x, y;
-	GList    *l, *to;
+	GList     *l, *to;
 	PrintTask *ptask;
 	gint       last, i;
 
@@ -249,10 +248,10 @@ print_table_tasks (MgGanttPrintData *data,
 		}
 
 		planner_print_job_show_clipped (data->job,
-					   x, y,
-					   str,
-					   data->name_x1, 0, 
-					   data->name_x2 - data->job->x_pad / 2, y + data->row_height);
+						x, y,
+						str,
+						data->name_x1, 0, 
+						data->name_x2 - data->job->x_pad / 2, y + data->row_height);
 
 		g_free (str);
 		
@@ -260,10 +259,10 @@ print_table_tasks (MgGanttPrintData *data,
 		
 		str = planner_format_duration (work, 8);
 		planner_print_job_show_clipped (data->job,
-					   x, y,
-					   str,
-					   data->work_x1, 0, 
-					   data->work_x2 - data->job->x_pad / 2, y + data->row_height);
+						x, y,
+						str,
+						data->work_x1, 0, 
+						data->work_x2 - data->job->x_pad / 2, y + data->row_height);
 		g_free (str);
 
 		y += data->row_height / 4;
@@ -280,11 +279,11 @@ print_table_tasks (MgGanttPrintData *data,
 }
 
 static void
-print_time_header (MgGanttPrintData *data,
-		   gdouble           x1,
-		   gdouble           x2,
-		   mrptime           start,
-		   mrptime           finish)
+print_time_header (PlannerGanttPrintData *data,
+		   gdouble                x1,
+		   gdouble                x2,
+		   mrptime                start,
+		   mrptime                finish)
 {
 	gdouble  x, y;
 	gdouble  y1, y2, y3;
@@ -326,10 +325,10 @@ print_time_header (MgGanttPrintData *data,
 			str = planner_scale_format_time (t, data->major_unit, data->major_format);
 			
 			planner_print_job_show_clipped (data->job,
-						   x + data->job->x_pad, y,
-						   str,
-						   MAX (x, x1), y1, 
-						   x + width, y2);
+							x + data->job->x_pad, y,
+							str,
+							MAX (x, x1), y1, 
+							x + width, y2);
 			
 			g_free (str);
 		}
@@ -357,10 +356,10 @@ print_time_header (MgGanttPrintData *data,
 			str = planner_scale_format_time (t, data->minor_unit, data->minor_format);
 			
 			planner_print_job_show_clipped (data->job,
-						   x + data->job->x_pad, y,
-						   str,
-						   MAX (x, x1), y2, 
-						   x + width, y3);
+							x + data->job->x_pad, y,
+							str,
+							MAX (x, x1), y2, 
+							x + width, y3);
 			
 			g_free (str);
 		}
@@ -371,7 +370,7 @@ print_time_header (MgGanttPrintData *data,
 
 typedef struct {
 	GtkTreeView *tree_view;
-	GList      *list;
+	GList       *list;
 } ForeachVisibleData;
 
 static gboolean
@@ -407,7 +406,7 @@ foreach_visible_task (GtkTreeModel *model,
 }
 
 static GList *
-gantt_print_get_visible_tasks (MgGanttPrintData *data)
+gantt_print_get_visible_tasks (PlannerGanttPrintData *data)
 {
 	ForeachVisibleData  fvd;
 	GtkTreeModel       *model;
@@ -435,7 +434,7 @@ gantt_print_free_print_tasks (GList *tasks)
 }	
 
 static GList *
-gantt_print_get_relations (MgGanttPrintData *data)
+gantt_print_get_relations (PlannerGanttPrintData *data)
 {
 	GList   *tasks, *l;
 	GList   *predecessors, *p;
@@ -456,7 +455,7 @@ gantt_print_get_relations (MgGanttPrintData *data)
 }
 
 static void
-gantt_print_task (MgGanttPrintData *data, Element *element)
+gantt_print_task (PlannerGanttPrintData *data, Element *element)
 {
 	gnome_print_newpath (data->job->pc);
 	planner_print_job_moveto (data->job, element->x1, element->y1);
@@ -465,26 +464,46 @@ gantt_print_task (MgGanttPrintData *data, Element *element)
 	planner_print_job_lineto (data->job, element->x1, element->y2);
 	gnome_print_closepath (data->job->pc);
 	
+	gnome_print_gsave (data->job->pc);
+
 	if (data->show_critical && element->is_critical) {
-		gnome_print_gsave (data->job->pc);
 		gnome_print_setrgbcolor (data->job->pc, 205/255.0, 92/255.0, 92/255.0);
-		gnome_print_fill (data->job->pc);
-		gnome_print_grestore (data->job->pc);
-		gnome_print_stroke (data->job->pc);
 	} else {
-		gnome_print_gsave (data->job->pc);
 		gnome_print_setrgbcolor (data->job->pc, 235/255.0, 235/255.0, 235/255.0);
+	}
+
+	gnome_print_fill (data->job->pc);
+	gnome_print_grestore (data->job->pc);
+
+	gnome_print_stroke (data->job->pc);
+
+	/* Percent complete. */
+	if (element->x_complete > 0) {
+		gdouble pad;
+
+		pad = (element->y2 - element->y1) * 0.25;
+		
+		gnome_print_gsave (data->job->pc);
+
+		gnome_print_newpath (data->job->pc);
+		planner_print_job_moveto (data->job, element->x1, element->y1 + pad);
+		planner_print_job_lineto (data->job, element->x_complete, element->y1 + pad);
+		planner_print_job_lineto (data->job, element->x_complete, element->y2 - pad);
+		planner_print_job_lineto (data->job, element->x1, element->y2 - pad);
+		gnome_print_closepath (data->job->pc);
+		
+		gnome_print_setrgbcolor (data->job->pc, 135/255.0, 135/255.0, 135/255.0);
 		gnome_print_fill (data->job->pc);
+		
 		gnome_print_grestore (data->job->pc);
-		gnome_print_stroke (data->job->pc);
 	}
 }
 
 static gboolean
-gantt_print_get_allocated_resources_string (MgGanttPrintData  *data,
-					    MrpTask           *task,
-					    gchar            **str,
-					    gdouble           *width)
+gantt_print_get_allocated_resources_string (PlannerGanttPrintData  *data,
+					    MrpTask                *task,
+					    gchar                 **str,
+					    gdouble                *width)
 {
 	GList         *l;
 	GList         *assignments;
@@ -501,23 +520,35 @@ gantt_print_get_allocated_resources_string (MgGanttPrintData  *data,
 
 		resource = mrp_assignment_get_resource (assignment);
 		units = mrp_assignment_get_units (assignment);
-		
-		g_object_get (resource, "name", &name, NULL);
 
+		/* Use the resource short_name in preference to the resource
+		 * name.
+		 */
+		g_object_get (resource, 
+			      "short_name", &name, 
+			      NULL);
+				
 		if (name && name[0] == 0) {
 			g_free (name);
-			name = NULL;
+
+			g_object_get (resource, 
+				      "name", &name, 
+				      NULL);
+			
+			if (name && name[0] == 0) {
+				g_free (name);
+				
+				name = g_strdup (_("Unnamed"));
+			}
 		}
 
-		if (units != 100) {
-			name_unit = g_strdup_printf ("%s [%i]", name ? name : _("Unnamed"), units);
+ 		if (units != 100) {
+			name_unit = g_strdup_printf ("%s [%i]", name, units);
 		} else {
-			name_unit = g_strdup_printf ("%s", name ? name : _("Unnamed"));
+			name_unit = g_strdup_printf ("%s", name);
 		}
-
-		g_free (name);
-
-		if (text == NULL) { /* First resource */
+		
+		if (!text) { /* First resource */
 			text = g_strdup_printf ("%s", name_unit);
 			g_free (name_unit);
 			continue;
@@ -525,8 +556,10 @@ gantt_print_get_allocated_resources_string (MgGanttPrintData  *data,
 		
 		tmp_str = g_strdup_printf ("%s, %s", text, name_unit);
 		
+		g_free (name);
 		g_free (text);
 		g_free (name_unit);
+
 		text = tmp_str;
 	}
 
@@ -549,7 +582,7 @@ gantt_print_get_allocated_resources_string (MgGanttPrintData  *data,
 }
 
 void
-planner_gantt_print_do (MgGanttPrintData *data)
+planner_gantt_print_do (PlannerGanttPrintData *data)
 {
 	GList      *relations;
 	GList      *l;
@@ -558,8 +591,10 @@ planner_gantt_print_do (MgGanttPrintData *data)
 	gint         num_tasks;
 	mrptime      t1, t2, t0;
 	mrptime      start, finish;
+	mrptime      complete;
 	gboolean     is_summary;
 	gboolean     is_critical;
+	gboolean     boundary_overlap = FALSE;
 	MrpTaskType  type;
 	PrintTask   *ptask;
 	TaskCoord   *task_coord;
@@ -604,7 +639,9 @@ planner_gantt_print_do (MgGanttPrintData *data)
 			
 			start = mrp_task_get_work_start (ptask->task);
 			finish = mrp_task_get_finish (ptask->task);
-			
+			complete = start + (finish - start) *
+				mrp_task_get_percent_complete (ptask->task) / 100.0;
+
 			is_summary = mrp_task_get_n_children (ptask->task) > 0;
 
 			g_object_get (ptask->task,
@@ -625,8 +662,9 @@ planner_gantt_print_do (MgGanttPrintData *data)
 			y1 = y0 + data->row_height * (i + 1.5 * 0.25);
 			y2 = y1 + 0.75 * data->row_height;
 
+			
 			/* Loop through the columns that this task covers. */
-			while (t1 < finish) {
+			while (t1 <= finish) {
 				if (col == 0) {
 					/* Left-most column has the task tree. */
 					x0 = data->tree_x2;
@@ -661,6 +699,7 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					
 					element->x1 = x0 + (start - t1) / data->f;
 					element->x2 = data->job->width;
+					element->x_complete = x0 + (complete - t1) / data->f;
 				}
 				else if (start < t1 && finish >= t1 && finish <= t2) {
 					/* Right */
@@ -677,6 +716,7 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					
 					element->x1 = x0;
 					element->x2 = x0 + (finish - t1) / data->f;
+					element->x_complete = x0 + (complete - t1) / data->f;
 				}
 				else if (start >= t1 && finish <= t2) {
 					/* Whole */
@@ -693,6 +733,7 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					
 					element->x1 = x0 + (start - t1) / data->f;
 					element->x2 = x0 + (finish - t1) / data->f;
+					element->x_complete = x0 + (complete - t1) / data->f;
 				}
 				else if (start < t1 && finish > t2) {
 					/* Middle */
@@ -709,7 +750,9 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					
 					element->x1 = x0;
 					element->x2 = data->job->width;
+					element->x_complete = data->job->width;
 				} else {
+					d(g_print ("nothing"));
 					g_free (element);
 					element = NULL;
 				}
@@ -772,6 +815,15 @@ planner_gantt_print_do (MgGanttPrintData *data)
 				task_coord = g_hash_table_lookup (data->task_finish_coords,
 								  ptask->task);
 
+				/* FIXME: Remove this check when the scheduler
+				 * is fixed. We get a crash here if tasks get
+				 * really odd start/finish values. This
+				 * sometimes happens for complex projects now.
+				 */
+				if (!task_coord) {
+					goto fail;
+				}
+
 				element = g_new0 (Element, 1);
 				element->type = RESOURCES;
 
@@ -783,6 +835,8 @@ planner_gantt_print_do (MgGanttPrintData *data)
 				page = GET_PAGE (data, row, col - 1);
 				page->elements = g_list_prepend (page->elements, element);
 			}
+
+		fail:
 			
 			l = l->next;
 			if (!l) {
@@ -903,8 +957,10 @@ planner_gantt_print_do (MgGanttPrintData *data)
 	g_list_free (relations);
 
 	/* Third pass, add background shading for non-work intervals. */
-	t0 = mrp_time_align_day (data->start);
 	for (row = 0; row < data->rows_of_pages; row++) {
+
+		t0 = mrp_time_align_day (data->start);
+
 		if (row == 0) {
 			/* Top-most row has the header. */
 			y0 = data->header_height;
@@ -926,6 +982,7 @@ planner_gantt_print_do (MgGanttPrintData *data)
 						
 			/* Loop through the days between t0 and t2. */
 			t1 = t0;
+
 			while (t1 <= t2) {
 				day = mrp_calendar_get_day (calendar, t1, TRUE);
 				
@@ -938,18 +995,31 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					ival = l->data;
 
 					mrp_interval_get_absolute (ival,
-								   t1,
+								   mrp_time_align_day (t1),
 								   &ival_start,
 								   &ival_end);
 
-					if (planner_scale_conf[data->level].nonworking_limit <= ival_start - ival_prev) {
+					if (planner_scale_conf[data->level].nonworking_limit <= ival_start - ival_prev ||
+					    (boundary_overlap && ival_start >= t1)) {
+
+						boundary_overlap = FALSE;
+
+						/* Check for corner case of
+						 * non-working interval
+						 * overlapping page boundary.
+						 */
+
+						if (ival_start > t2) {
+							boundary_overlap = TRUE;
+						}
+
 						element = g_new0 (Element, 1);
 						element->type = SHADE;
 						element->y1 = y0 + data->row_height / 4;
 						element->y2 = data->job->height;
 
-						element->x1 = x0 + (ival_prev - data->start) / data->f;
-						element->x2 = x0 + (ival_start - data->start) / data->f;
+						element->x1 = x0 + (ival_prev - t0) / data->f;
+						element->x2 = x0 + (ival_start - t0) / data->f;
 						
 						page = GET_PAGE (data, row, col);
 						page->background_elements = g_list_prepend (page->background_elements, element);
@@ -959,21 +1029,37 @@ planner_gantt_print_do (MgGanttPrintData *data)
 				}
 
 				t1 += 60*60*24;
+				t1 =  mrp_time_align_day (t1);
 
 				/* Draw the remaining interval if there is one. */
-				if (ival_prev < t1 && planner_scale_conf[data->level].nonworking_limit <= t1 - ival_prev) {
+				if ((ival_prev < t1 && planner_scale_conf[data->level].nonworking_limit <= t1 - ival_prev) ||
+				    boundary_overlap) {
+
+					boundary_overlap = FALSE;
+
+					/* Check for corner case of non-working
+					 * interval overlapping page
+					 * boundary.
+					 */
+
+					if (t1 > t2) {
+						boundary_overlap = TRUE;
+					}
+
 					element = g_new0 (Element, 1);
 					element->type = SHADE;
 					element->y1 = y0 + data->row_height / 4;
 					element->y2 = data->job->height;
 					
-					element->x1 = x0 + (ival_prev - data->start) / data->f;
-					element->x2 = x0 + (t1 - data->start) / data->f;
+					element->x1 = x0 + (ival_prev - t0) / data->f;
+					element->x2 = x0 + (t1 - t0) / data->f;
 					
 					page = GET_PAGE (data, row, col);
 					page->background_elements = g_list_prepend (page->background_elements, element);
 				}
 			}
+
+			t0 = t2;
 		}
 	}
 	
@@ -991,24 +1077,24 @@ planner_gantt_print_do (MgGanttPrintData *data)
 				t2 = t1 + (data->job->width - (data->tree_x2 - data->tree_x1)) * data->f;
 
 				planner_print_job_moveto (data->job,
-						     data->tree_x2,
-						     0);
+							  data->tree_x2,
+							  0);
 				planner_print_job_lineto (data->job,
-						     data->tree_x2,
-						     data->job->height);
+							  data->tree_x2,
+							  data->job->height);
 				gnome_print_stroke (data->job->pc);
 				
 				planner_print_job_moveto (data->job,
-						     data->name_x2,
-						     0);
+							  data->name_x2,
+							  0);
 				planner_print_job_lineto (data->job,
-						     data->name_x2,
-						     data->job->height);
+							  data->name_x2,
+							  data->job->height);
 				gnome_print_stroke (data->job->pc);
 
 				print_table_tasks (data,
 						   row == 0,
-						   data->tasks, /* FIXME: remove, use data->tasks */
+						   data->tasks,
 						   i);
 			} else {
 				x1 = 0;
@@ -1065,139 +1151,139 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					break;
 				case SUMMARY_LEFT:
 					planner_print_job_moveto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick + data->summary_height);
+								  element->x1,
+								  element->y1 + data->summary_thick + data->summary_height);
 					planner_print_job_lineto (data->job,
-							     element->x1 + data->summary_slope,
-							     element->y1 + data->summary_thick);
+								  element->x1 + data->summary_slope,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 
 					planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x2,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x1,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case SUMMARY_RIGHT:
 					planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick + data->summary_height);
+								  element->x2,
+								  element->y1 + data->summary_thick + data->summary_height);
 					planner_print_job_lineto (data->job,
-							     element->x2 - data->summary_slope,
-							     element->y1 + data->summary_thick);
+								  element->x2 - data->summary_slope,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 
 					planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x2,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x1,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case SUMMARY_WHOLE:
 					planner_print_job_moveto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick + data->summary_height);
+								  element->x1,
+								  element->y1 + data->summary_thick + data->summary_height);
 					planner_print_job_lineto (data->job,
-							     element->x1 + data->summary_slope,
-							     element->y1 + data->summary_thick);
+								  element->x1 + data->summary_slope,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					
 					planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick + data->summary_height);
+								  element->x2,
+								  element->y1 + data->summary_thick + data->summary_height);
 					planner_print_job_lineto (data->job,
-							     element->x2 - data->summary_slope,
-							     element->y1 + data->summary_thick);
+								  element->x2 - data->summary_slope,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					
 					planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x2,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x1,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case SUMMARY_MIDDLE:
-						planner_print_job_moveto (data->job,
-							     element->x2,
-							     element->y1 + 2 * data->summary_thick);
+					planner_print_job_moveto (data->job,
+								  element->x2,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + 2 * data->summary_thick);
+								  element->x1,
+								  element->y1 + 2 * data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + data->summary_thick);
+								  element->x1,
+								  element->y1 + data->summary_thick);
 					planner_print_job_lineto (data->job,
-							     element->x2,
-							     element->y1 + data->summary_thick);
+								  element->x2,
+								  element->y1 + data->summary_thick);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case RELATION_ARROW_DOWN:
 					planner_print_job_moveto (data->job,
-							     element->x1,
-							     element->y1);
+								  element->x1,
+								  element->y1);
 					planner_print_job_lineto (data->job,
-							     element->x1 - data->arrow_width,
-							     element->y1 - data->arrow_height);
+								  element->x1 - data->arrow_width,
+								  element->y1 - data->arrow_height);
 					planner_print_job_lineto (data->job,
-							     element->x1 + data->arrow_width,
-							     element->y1 - data->arrow_height);
+								  element->x1 + data->arrow_width,
+								  element->y1 - data->arrow_height);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case RELATION_ARROW_UP:
 					planner_print_job_moveto (data->job,
-							     element->x1,
-							     element->y1);
+								  element->x1,
+								  element->y1);
 					planner_print_job_lineto (data->job,
-							     element->x1 - data->arrow_width,
-							     element->y1 + data->arrow_height);
+								  element->x1 - data->arrow_width,
+								  element->y1 + data->arrow_height);
 					planner_print_job_lineto (data->job,
-							     element->x1 + data->arrow_width,
-							     element->y1 + data->arrow_height);
+								  element->x1 + data->arrow_width,
+								  element->y1 + data->arrow_height);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
@@ -1213,28 +1299,28 @@ planner_gantt_print_do (MgGanttPrintData *data)
 					break;
 				case MILESTONE:
 					planner_print_job_moveto (data->job,
-							     element->x1,
-							     element->y1);
+								  element->x1,
+								  element->y1);
 					planner_print_job_lineto (data->job,
-							     element->x1 + data->milestone_size,
-							     element->y1 + data->milestone_size);
+								  element->x1 + data->milestone_size,
+								  element->y1 + data->milestone_size);
 					planner_print_job_lineto (data->job,
-							     element->x1,
-							     element->y1 + 2 * data->milestone_size);
+								  element->x1,
+								  element->y1 + 2 * data->milestone_size);
 					planner_print_job_lineto (data->job,
-							     element->x1 - data->milestone_size,
-							     element->y1 + data->milestone_size);
+								  element->x1 - data->milestone_size,
+								  element->y1 + data->milestone_size);
 					gnome_print_closepath (data->job->pc);
 					gnome_print_fill (data->job->pc);
 					break;
 				case RESOURCES:
 					planner_print_job_show_clipped (data->job,
-								   element->x1,
-								   element->y1,
-								   element->resources,
-								   0, 0,
-								   data->job->width,
-								   data->job->height);
+									element->x1,
+									element->y1,
+									element->resources,
+									0, 0,
+									data->job->width,
+									data->job->height);
 					break;
 				default:
 					break;
@@ -1253,24 +1339,24 @@ planner_gantt_print_do (MgGanttPrintData *data)
 	}
 }
 
-MgGanttPrintData *
-planner_gantt_print_data_new (MgView      *view,
-			 MgPrintJob  *job,
-			 GtkTreeView *tree_view,
-			 gint         level,
-			 gboolean     show_critical)
+PlannerGanttPrintData *
+planner_gantt_print_data_new (PlannerView     *view,
+			      PlannerPrintJob *job,
+			      GtkTreeView     *tree_view,
+			      gint             level,
+			      gboolean         show_critical)
 {
-	MgGanttPrintData *data;
-	GnomeFont        *font;
-	GList            *tasks = NULL, *l;
-	gint              num_tasks;
-	gdouble           max_name_width = 0.0;
+	PlannerGanttPrintData *data;
+	GnomeFont             *font;
+	GList                 *tasks = NULL, *l;
+	gint                   num_tasks;
+	gdouble                max_name_width = 0.0;
 
-	data = g_new0 (MgGanttPrintData, 1);
+	data = g_new0 (PlannerGanttPrintData, 1);
 
 	data->view = view;
 	data->job = job;
-	data->project = planner_main_window_get_project (view->main_window);
+	data->project = planner_window_get_project (view->main_window);
 
 	data->tree_view = tree_view;
 
@@ -1279,7 +1365,7 @@ planner_gantt_print_data_new (MgView      *view,
 	
 	/* Note: This looks hackish, but it's the same equation used for the
 	 * zoom level in the gantt chart, which actually is calculated to have a
-	 * "good feel" :). We scale it width the paper size here though...
+	 * "good feel" :). We scale it with the paper size here though...
 	 */
 	data->f = 1000 / pow (2, level - 19) / data->job->width;
 
@@ -1291,8 +1377,8 @@ planner_gantt_print_data_new (MgView      *view,
 
 	font = planner_print_job_get_font (job);
 	
-	data->task_start_coords = g_hash_table_new (NULL, NULL);
-	data->task_finish_coords = g_hash_table_new (NULL, NULL);
+	data->task_start_coords = g_hash_table_new_full (NULL, NULL, NULL, g_free);
+	data->task_finish_coords = g_hash_table_new_full (NULL, NULL, NULL, g_free);
 
 	/* Start and finish of the project. */
 	data->start = mrp_project_get_project_start (data->project);
@@ -1313,26 +1399,29 @@ planner_gantt_print_data_new (MgView      *view,
 		mrptime    finish;
 		gdouble    width;
 		gdouble    name_width;
-		gchar     *str;
 		
-		g_object_get (task, "name", &name, "finish", &finish, NULL);
+		g_object_get (task,
+			      "name", &name,
+			      "finish", &finish,
+			      NULL);
 
 		name_width = gnome_font_get_width_utf8 (font, name) + 
-			       ptask->depth * INDENT_FACTOR * data->job->x_pad;
-		if (max_name_width < name_width)
+			ptask->depth * INDENT_FACTOR * data->job->x_pad;
+		
+		if (max_name_width < name_width) {
 			max_name_width = name_width;
+		}
 
-		gantt_print_get_allocated_resources_string (data, task, &str, &width);
+		gantt_print_get_allocated_resources_string (data, task, NULL, &width);
 
 		data->finish = MAX (data->finish, finish);/* + width * data->f);*/
 	}
-
+	
 	data->name_x1 = 0;
-	data->name_x2 = data->name_x1 + max_name_width + gnome_font_get_width_utf8
-		(font, "WW");
+	data->name_x2 = data->name_x1 + max_name_width + gnome_font_get_width_utf8 (font, "WW");
 	
 	data->work_x1 = data->name_x2;
-	data->work_x2 = data->work_x1 + gnome_font_get_width_utf8 (font, "WORKW");
+	data->work_x2 = data->work_x1 + gnome_font_get_width_utf8 (font, "WORKWO");
 
 	data->tree_x1 = 0;
 	data->tree_x2 = data->work_x2;
@@ -1348,16 +1437,16 @@ planner_gantt_print_data_new (MgView      *view,
 	data->milestone_size = 0.40 * data->row_height;
 	data->arrow_height   = 0.24 * data->row_height;
 	data->arrow_width    = 0.16 * data->row_height;
-
+	
 	if (num_tasks > 0) {
-		data->tasks_per_page_with_header = data->job->height / data->row_height;
-		data->tasks_per_page_without_header = (data->job->height - data->header_height) /
+		data->tasks_per_page_without_header = data->job->height / data->row_height;
+		data->tasks_per_page_with_header = (data->job->height - data->header_height) /
 			data->row_height;
-
+		
 		data->cols_of_pages = ceil (((data->finish - data->start) /
 					     data->f + data->tree_x2 - data->tree_x1) /
 					    data->job->width);
-
+		
 		data->rows_of_pages = ceil ((num_tasks * data->row_height + data->header_height) /
 					    (data->job->height - data->row_height));
  		
@@ -1365,7 +1454,7 @@ planner_gantt_print_data_new (MgView      *view,
 		    data->tasks_per_page_with_header >= num_tasks) {
 			data->rows_of_pages--;
 		}
-
+		
 		data->cols_of_pages = MAX (1, data->cols_of_pages);
 		data->rows_of_pages = MAX (1, data->rows_of_pages);
 		
@@ -1375,9 +1464,31 @@ planner_gantt_print_data_new (MgView      *view,
 	return data;
 }
 
-void
-planner_gantt_print_data_free (MgGanttPrintData *data)
+static void
+free_page (Page *page)
 {
+	GList   *l;
+	Element *element;
+
+	for (l = page->elements; l; l = l->next) {
+		element = l->data;
+		g_free (element->resources);
+	}
+
+	for (l = page->background_elements; l; l = l->next) {
+		element = l->data;
+		g_free (element->resources);
+	}
+
+	g_list_free (page->elements);
+	g_list_free (page->background_elements);
+}
+
+void
+planner_gantt_print_data_free (PlannerGanttPrintData *data)
+{
+	gint i, num_pages;
+	
 	g_return_if_fail (data != NULL);
 
 	g_hash_table_destroy (data->task_start_coords);
@@ -1386,6 +1497,12 @@ planner_gantt_print_data_free (MgGanttPrintData *data)
 	gantt_print_free_print_tasks (data->tasks);
 	data->tasks = NULL;
 
+	num_pages = data->cols_of_pages * data->rows_of_pages;
+	
+	for (i = 0; i < num_pages; i++) {
+		free_page (&data->pages[i]);
+	}
+	
 	g_free (data->pages);
 	data->pages = NULL;
 
@@ -1393,7 +1510,7 @@ planner_gantt_print_data_free (MgGanttPrintData *data)
 }
 
 gint
-planner_gantt_print_get_n_pages (MgGanttPrintData *data)
+planner_gantt_print_get_n_pages (PlannerGanttPrintData *data)
 {
 	g_return_val_if_fail (data != NULL, 0);
 	
